import { AlgorandClient } from '../../types/algorand-client.mjs';
import { ClientManager } from '../../types/client-manager.mjs';
import { getTestAccount } from '../account.mjs';
import { runWhenIndexerCaughtUp } from '../indexer.mjs';
import { TransactionLogger } from '../transaction-logger.mjs';
import { algos } from '../../amount.mjs';
import { Config } from '../../config.mjs';

function algorandFixture(fixtureConfig, config) {
    fixtureConfig = { ...fixtureConfig, ...config };
    if (!fixtureConfig.algod || !fixtureConfig.indexer || !fixtureConfig.kmd) {
        fixtureConfig = { ...ClientManager.getConfigFromEnvironmentOrLocalNet(), ...fixtureConfig };
    }
    const algod = fixtureConfig.algod ?? ClientManager.getAlgodClient(fixtureConfig.algodConfig);
    const indexer = fixtureConfig.indexer ?? ClientManager.getIndexerClient(fixtureConfig.indexerConfig);
    const kmd = fixtureConfig.kmd ?? ClientManager.getKmdClient(fixtureConfig.kmdConfig);
    let context;
    let algorand;
    const newScope = async () => {
        Config.configure({ debug: true });
        const transactionLogger = new TransactionLogger();
        const transactionLoggerAlgod = transactionLogger.capture(algod);
        algorand = AlgorandClient.fromClients({ algod: transactionLoggerAlgod, indexer, kmd }).setSuggestedParamsCacheTimeout(0);
        const testAccount = await getTestAccount({ initialFunds: fixtureConfig?.testAccountFunding ?? algos(10), suppressLog: true }, algorand);
        algorand.setSignerFromAccount(testAccount);
        // If running against LocalNet we are likely in dev mode and we need to set a much higher validity window
        //  otherwise we are more likely to get invalid transactions.
        if (await algorand.client.isLocalNet()) {
            algorand.setDefaultValidityWindow(1000);
        }
        context = {
            algorand,
            algod: transactionLoggerAlgod,
            indexer: indexer,
            kmd: kmd,
            testAccount,
            generateAccount: async (params) => {
                const account = await getTestAccount(params, algorand);
                algorand.setSignerFromAccount(account);
                return account;
            },
            transactionLogger: transactionLogger,
            waitForIndexer: () => transactionLogger.waitForIndexer(indexer),
            waitForIndexerTransaction: (transactionId) => runWhenIndexerCaughtUp(() => indexer.lookupTransactionByID(transactionId).do()),
        };
    };
    return {
        get context() {
            if (!context)
                throw new Error('Context not initialised; make sure to call fixture.newScope() before accessing context.');
            return context;
        },
        get algorand() {
            return algorand;
        },
        beforeEach: newScope,
        newScope,
    };
}

export { algorandFixture };
//# sourceMappingURL=algorand-fixture.mjs.map
